/*
' COPanno Optimized rectangular cutting tool
'
'
' Copyright (C)2012-2015 Florian Foinant
'
' This program is free software; you can redistribute it and/or modify
' it under the terms of the GNU General Public License as published by
' the Free Software Foundation; either version 2 of the License, or
' (at your option) any later version.
'
' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 51 Franklin St, Fifth Floor,
' Boston, MA  02110-1301  USA
'*/

import java.awt.*;
import java.awt.event.*;
import java.awt.print.*;
import java.io.*;
import java.util.ArrayList;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.table.AbstractTableModel;

class GUI {
	JFrame mainFrame = null;

	//Contrôles dans la feuille
	JCanvas jc;
	JSpinner SpinBox1;
	JSpinner SpinBox2;
	JTable jt;
	JLabel jLabel0;
	JCheckBox jCheck0;
	JCheckBox jCheck1;
	JProgressBar jProgress;
	JButton btn;
	JButton btnPrint;
	JButton btnReset;
	JComboBox<String> combo1;
	JPanel hp;
	JPanel rp;
	
	//Menu fichier
	JMenuBar menuBar = new JMenuBar();
	JMenu menuFichier = new JMenu(I18N.getString("Fichier"));
	JMenuItem menuItemFichier_Ouvrir = new JMenuItem(I18N.getString("Ouvrir"));
	JMenuItem menuItemFichier_Enregistrer = new JMenuItem(I18N.getString("Enregistrer"));
	JMenuItem menuItemFichier_Exporter = new JMenuItem(I18N.getString("Exporter"));
	JMenuItem menuItemFichier_Fermer = new JMenuItem(I18N.getString("Fermer"));

	//Données de la table
	Object[][] rawdata = new Object[1000][5];

	//Objets pour le calcul
	Solution solution;
	Panneau feuille = new Panneau(2500, 1250);
	ArrayList<Panneau> debit;
	JProgressBar progress;

	@SuppressWarnings("serial")
	public GUI() {

		mainFrame = new JFrame();
		mainFrame.setTitle("COPanno");

		mainFrame.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
		mainFrame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent ev) {
		  		JOptionPane jop=new JOptionPane();
				int option = jop.showConfirmDialog(null, I18N.getString("quitter?"), I18N.getString("Confirmation"), JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
				if(option == JOptionPane.OK_OPTION){
					System.exit(0);
				}
			}
		});

		//Définition de la géométrie
		mainFrame.setSize(800, 800);

		//XXX Initialisation des contrôles
		//Menu fichier
		menuItemFichier_Ouvrir.addActionListener(new ActionListener(){
	      public void actionPerformed(ActionEvent event){
	    	  
	      }
		});
		menuFichier.add(menuItemFichier_Ouvrir);
		menuFichier.add(menuItemFichier_Enregistrer);
		menuFichier.add(menuItemFichier_Exporter);
		menuFichier.addSeparator();
		menuItemFichier_Fermer.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent event){
				JOptionPane jop=new JOptionPane();
				int option = jop.showConfirmDialog(null, I18N.getString("quitter?"), I18N.getString("Confirmation"), JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
				if(option == JOptionPane.OK_OPTION){
					System.exit(0);
				}
		      }
		    });
		menuFichier.add(menuItemFichier_Fermer);
		
		//Barre de menu
		menuBar.add(menuFichier);
		mainFrame.setJMenuBar(menuBar);
		
		//SpinBox1 Feuille à afficher
		SpinBox1 = new JSpinner(new SpinnerNumberModel(0, 0, 1, 1));
		SpinBox1.addChangeListener(new ChangeListener(){
			@Override
			public void stateChanged( ChangeEvent e ) {
				jc.repaint();
			}
		});
		//SpinBox2 Taille du trait de coupe
		SpinBox2 = new JSpinner(new SpinnerNumberModel(3, 0, 10, 0.1));

		//XXX Bouton calculer
		btn = new JButton(I18N.getString("Calculer"));

		ActionListener actCalculer=new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				int row=0;
				try {
					if (debit != null)
						debit.clear();
					else
						debit = new ArrayList<Panneau>();

					double coupe =  Double.parseDouble(SpinBox2.getValue().toString());

					String[] format = ((String)combo1.getEditor().getItem()).split("x");
					feuille=new Panneau(Double.parseDouble(format[0])+coupe ,Double.parseDouble(format[1])+coupe);

					for (row=0 ; row<jt.getRowCount() ; row++)
						if (jt.getValueAt(row, 0) != null){
							debit.add( new Panneau((Double)jt.getValueAt(row, 2)+coupe,
									(Double)jt.getValueAt(row, 3)+coupe,
									(String)jt.getValueAt(row, 0),
									(Integer)jt.getValueAt(row, 1),
									(Boolean)jt.getValueAt(row, 4) ));
						}

					Thread t = new Thread() {
						public void run() {
							mainFrame.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
							for (int n=0 ; n<hp.getComponentCount(); n++){
								hp.getComponent(n).setEnabled(false);
							}
							for (int n=0 ; n<rp.getComponentCount(); n++){
								rp.getComponent(n).setEnabled(false);
							}
							jt.setEnabled(false);
							jProgress.setString(null);
							jProgress.setValue(0);
							
							Solver solv=new Solver(feuille, debit, jCheck0.isSelected());
							solv.go();
							int t;
							while((t=solv.isRunning())>0){
								jProgress.setValue(8-t);
							}
							setSolution(solv.getSolution());

							jProgress.setValue(0);
							jProgress.setString("Terminé");
							jt.setEnabled(true);
							for (int n=0 ; n<hp.getComponentCount(); n++){
								hp.getComponent(n).setEnabled(true);
							}
							for (int n=0 ; n<rp.getComponentCount(); n++){
								rp.getComponent(n).setEnabled(true);
							}
							mainFrame.setCursor(null);
						}
					};
					t.start();
				}
				catch(Exception err){
					JOptionPane.showMessageDialog(null, String.format(I18N.getString("Erreur_format"),row+1));
				}
			}
		};

		btn.addActionListener(actCalculer);

		//XXX Bouton imprimer
		btnPrint = new JButton("Imprimer");
		btnPrint.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				PrinterJob pj = PrinterJob.getPrinterJob();
				pj.setJobName("COPanno print");
				pj.setCopies(1);
				PageFormat format = pj.defaultPage();
				format.setOrientation(PageFormat.PORTRAIT);

				pj.setPrintable(new Printable() {
					public int print(Graphics pg, PageFormat pf, int pageNum){
						if (pageNum > ((feuille.hauteur * 0.15 + 10) * solution.nbrFeuille() / pf.getImageableHeight())){
							return Printable.NO_SUCH_PAGE;
						}

						Graphics2D g2 = (Graphics2D) pg;
						g2.setBackground(Color.white);
						g2.translate(pf.getImageableX(), pf.getImageableY());
						jc.print(g2, pageNum);
						return Printable.PAGE_EXISTS;
					}
				});
				if (pj.printDialog() == false)
					return;

				try {
					pj.print();
				} catch (PrinterException e1) {
					e1.printStackTrace();
				}
			}
		});

		//Bouton reset
		btnReset = new JButton(I18N.getString("Reinitialiser"));
		btnReset.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				rawdata= new Object[1000][7];
				jt.repaint();
			}
		});

		////
		//Choix de la feuille

		//Liste de base
		if (! new File(_H.SheetsFile).exists()){
			String[] tmp_lst={"2500x1220", "2440x1220","2500x1250", "2440x1250","3050x1530", "2800x2070",
					"2800x1196", "2400x675", "2500x675","2400x600","2000x910","2050x925","2400x1205","5000x1000",
					"5000x1250","5000x2000", "5000x2050","2035x905"};
			try {
				//enregistre les dimensions de feuilles dans le fichier
				PrintWriter out = new PrintWriter(new BufferedWriter( new FileWriter(_H.SheetsFile)));
				for (int i=0; i<tmp_lst.length; i++){
					out.write(tmp_lst[i]);
					out.println();
				}
				out.flush();
				out.close();
			} catch (IOException e1) {
				e1.printStackTrace();
			}
		}

		final ArrayList<String> predef = new ArrayList<String>();

		//Charge les dimensions de feuilles depuis le fichier
		try {
			BufferedReader in = new BufferedReader(new FileReader(_H.SheetsFile));
			String ln;
			while ((ln=in.readLine()) != null)
				predef.add(ln);
			in.close();
		}catch (IOException e){
			e.printStackTrace();
		}
		
		//Liste déroulante des dimensions de feuilles
		combo1 = new JComboBox(predef.toArray());
		combo1.setEditable(true);
		combo1.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				//lors de l'édition
				JComboBox cb = (JComboBox)e.getSource();
				String format = (String)cb.getEditor().getItem();
				if (format.indexOf("x") != -1){
					//Enregistre la nouvelle feuille
					if (predef.indexOf(format)==-1){
						predef.add(format);
						try {
							PrintWriter out = new PrintWriter(new BufferedWriter( new FileWriter(_H.SheetsFile, true)));
							out.write(format);
							out.println();
							out.flush();
							out.close();
						} catch (IOException e1) {
							e1.printStackTrace();
						}
					}
				}
				else
					JOptionPane.showMessageDialog(null,I18N.getString("Format_incorrect"));
			}
		});

		//XXX Barre du haut
		hp= new JPanel();
		hp.add(new JLabel(I18N.getString("Taille_feuille")));
		hp.add(combo1);
		jCheck0 = new JCheckBox(I18N.getString("Methode_simplifiee"),false);
		hp.add(jCheck0);
		hp.add(new JPanel());
		hp.add(new JLabel(I18N.getString("Largeur_coupe")));
		hp.add(SpinBox2); 
		jProgress=new JProgressBar();
		jProgress.setMaximum(8);
		hp.add(jProgress);
		mainFrame.getContentPane().add(hp, BorderLayout.NORTH);

		//XXX Barre de droite
		rp = new JPanel();
		rp.setLayout(new BoxLayout(rp,BoxLayout.PAGE_AXIS));
		rp.add(btnReset);
		rp.add(btn);
		rp.add(btnPrint);
		rp.add(new JLabel(I18N.getString("Afficher_feuille")));
		JPanel srp=new JPanel();
		srp.setMaximumSize(new Dimension(80,20));
		srp.setLayout(new BoxLayout(srp,BoxLayout.LINE_AXIS));
		srp.add(SpinBox1);
		jLabel0 = new JLabel(String.format(I18N.getString("sur"),0));
		srp.add(jLabel0);
		rp.add(srp);
		mainFrame.getContentPane().add(rp, BorderLayout.EAST);

		// Table
		jt = new JTable();
		jt.setModel(new AbstractTableModel(){
			public String getColumnName(int col) {
				String[] names = {I18N.getString("Repere"),I18N.getString("Quantite"),
						I18N.getString("Largeur"),I18N.getString("Hauteur"),I18N.getString("Garder_fil")};
				return names[col];
			}
			@Override
            public Class getColumnClass(int column) {
                switch (column) {
                    case 0:
                        return String.class;
                    case 1:
                        return Integer.class;
                    case 2:
                        return Double.class;
                    case 3:
                        return Double.class;
                    default:
                        return Boolean.class;
                }
            }
			public int getColumnCount() {
				return 5;
			}
			public int getRowCount() {
				return 100;
			}
			public Object getValueAt(int row, int col) {
				if (row < rawdata.length){
					if (row==jt.getEditingRow() && col==jt.getEditingColumn())
						jt.getCellEditor().stopCellEditing();
					if (col == 4){
						if (rawdata[row][4]==null)
							rawdata[row][4]=false;
					}
					return rawdata[row][col];
				}else
					return null;
			}
			public boolean isCellEditable(int row, int col) {
				return true;
			}
			public void setValueAt(Object aValue, int row, int col){
				rawdata[row][col]=aValue;
				jt.repaint();
			}
		});
		jt.setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
		jt.setRowSelectionAllowed(false);
		jt.setColumnSelectionAllowed(false);
		jt.setCellSelectionEnabled(true);

		JScrollPane scrollpane = new JScrollPane(jt);
		mainFrame.getContentPane().add(scrollpane, BorderLayout.CENTER);

		// Panneau d'affichage
		jc = new JCanvas(solution, SpinBox1, SpinBox2, jLabel0);
		jc.setBackground(Color.WHITE);
		jc.setPreferredSize(new Dimension(400,600));
		mainFrame.getContentPane().add(jc,BorderLayout.SOUTH);

		//Prise en charge du presse papier
		@SuppressWarnings("unused")
		ExcelAdapter myAd = new ExcelAdapter(jt);

		mainFrame.setExtendedState(Frame.MAXIMIZED_BOTH);
	}

	public void afficher() {
		mainFrame.setVisible(true);
	}

	public void setTitle(String title) {
		mainFrame.setTitle(title);
	}

	public void setSolution(final Solution sol){
		solution= sol;

		jc.solution = sol;
		jc.feuille = feuille;
		jc.repaint(1);
	}

	public void setFeuille(Panneau feui){
		feuille= feui;
	}
}